@@ -1,6 +1,19 @@
-# file MASS/lda.q
+# file MASS/R/lda.R
 # copyright (C) 1994-2004 W. N. Venables and B. D. Ripley
 #
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 or 3 of the License
+#  (at your option).
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+#
 lda <- function(x, ...) UseMethod("lda")
 
 
@@ -66,6 +79,8 @@ lda.default <-
 {
     if(is.null(dim(x))) stop("'x' is not a matrix")
     x <- as.matrix(x)
+    if(any(!is.finite(x)))
+        stop("infinite, NA or NaN values in 'x'")
     n <- nrow(x)
     p <- ncol(x)
     if(n != length(grouping))
@@ -99,8 +114,8 @@ lda.default <-
     if(any(f1 < tol)) {
         const <- format((1:p)[f1 < tol])
         stop(sprintf(ngettext(length(const),
-                     "variable %d appears to be constant within groups",
-                     "variables %d appear to be constant within groups"),
+                     "variable %s appears to be constant within groups",
+                     "variables %s appear to be constant within groups"),
                      paste(const, collapse = " ")),
              domain = NA)
     }
@@ -111,6 +126,7 @@ lda.default <-
         cov <- n/(n-ng) * cov.rob((x - group.means[g,  ]) %*% scaling)$cov
         sX <- svd(cov, nu = 0)
         rank <- sum(sX$d > tol^2)
+        if(rank == 0) stop("rank = 0: variables are numerically const")
         if(rank < p) warning("variables are collinear")
         scaling <- scaling %*% sX$v[, 1:rank] %*%
             diag(sqrt(1/sX$d[1:rank]),,rank)
@@ -125,12 +141,13 @@ lda.default <-
             w <- 1/(1 + drop(X^2 %*% rep(1, p))/nu)
             print(summary(w))
             group.means <- tapply(w*x, list(rep(g, p), col(x)), sum)/
-                rep(tapply(w, g, sum), p)
+                rep.int(tapply(w, g, sum), p)
             if(all(abs(w - w0) < 1e-2)) break
         }
         X <-  sqrt(nu/(nu-2)*(1 + p/nu)/n * w) * (x - group.means[g,  ]) %*% scaling
         X.s <- svd(X, nu = 0)
         rank <- sum(X.s$d > tol)
+        if(rank == 0) stop("rank = 0: variables are numerically const")
         if(rank < p) warning("variables are collinear")
         scaling <- scaling %*% X.s$v[, 1:rank] %*% diag(1/X.s$d[1:rank],,rank)
     } else {
@@ -138,6 +155,7 @@ lda.default <-
         X <- sqrt(fac) * (x - group.means[g,  ]) %*% scaling
         X.s <- svd(X, nu = 0)
         rank <- sum(X.s$d > tol)
+        if(rank == 0) stop("rank = 0: variables are numerically const")
         if(rank < p) warning("variables are collinear")
         scaling <- scaling %*% X.s$v[, 1:rank] %*% diag(1/X.s$d[1:rank],,rank)
     }
@@ -176,6 +194,7 @@ lda.default <-
     X <- sqrt((n * prior)*fac) * scale(group.means, center=xbar, scale=FALSE) %*% scaling
     X.s <- svd(X, nu = 0)
     rank <- sum(X.s$d > tol * X.s$d[1])
+    if(rank == 0) stop("group means are numerically identical")
     scaling <- scaling %*% X.s$v[, 1:rank]
     if(is.null(dimnames(x)))
         dimnames(scaling) <- list(NULL, paste("LD", 1:rank, sep = ""))
@@ -202,8 +221,7 @@ predict.lda <- function(object, newdata, prior = object$prior, dimen,
         else {
             newdata <- model.frame(Terms, newdata, na.action=na.pass,
                                    xlev = object$xlevels)
-            if (!is.null(cl <- attr(Terms, "dataClasses")) &&
-                exists(".checkMFClasses", envir=NULL))
+            if (!is.null(cl <- attr(Terms, "dataClasses")))
                 .checkMFClasses(cl, newdata)
         }
         x <- model.matrix(Terms, newdata, contrasts = object$contrasts)
@@ -280,7 +298,7 @@ print.lda <- function(x, ...)
     if(!is.null(cl <- x$call)) {
         names(cl)[2] <- ""
         cat("Call:\n")
-        dput(cl)
+        dput(cl, control=NULL)
     }
     cat("\nPrior probabilities of groups:\n")
     print(x$prior, ...)
@@ -454,7 +472,7 @@ pairs.lda <- function(x, labels = colnames(x), panel = panel.lda,
                              key = list(
                              text=list(levels(g)),
                              points = lattice::Rows(lattice::trellis.par.get("superpose.symbol"),
-                             seq(along=levels(g))),
+                             seq_along(levels(g))),
                              columns = min(5, length(levels(g)))
                              )
                     ))
